@@ -26,20 +26,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkctree.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvpaned.h>
-#include <gtk/gtktext.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkmenu.h>
-#include <gtk/gtkdnd.h>
-#include <gtk/gtkselection.h>
-#include <gtk/gtktooltips.h>
-#include <gtk/gtkcontainer.h>
-#include <gtk/gtkbutton.h>
+#include <gtk/gtk.h>
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -88,10 +75,10 @@ typedef enum
 
 static void mimeview_set_multipart_tree		(MimeView	*mimeview,
 						 MimeInfo	*mimeinfo,
-						 GtkCTreeNode	*parent);
-static GtkCTreeNode *mimeview_append_part	(MimeView	*mimeview,
+						 GtkCMCTreeNode	*parent);
+static GtkCMCTreeNode *mimeview_append_part	(MimeView	*mimeview,
 						 MimeInfo	*partinfo,
-						 GtkCTreeNode	*parent);
+						 GtkCMCTreeNode	*parent);
 static void mimeview_show_message_part		(MimeView	*mimeview,
 						 MimeInfo	*partinfo);
 static void mimeview_change_view_type		(MimeView	*mimeview,
@@ -102,8 +89,8 @@ static gchar *mimeview_get_filename_for_part		(MimeInfo	*partinfo,
 static gboolean mimeview_write_part		(const gchar	*filename,
 						 MimeInfo	*partinfo);
 
-static void mimeview_selected		(GtkCTree	*ctree,
-					 GtkCTreeNode	*node,
+static void mimeview_selected		(GtkCMCTree	*ctree,
+					 GtkCMCTreeNode	*node,
 					 gint		 column,
 					 MimeView	*mimeview);
 static void mimeview_start_drag 	(GtkWidget	*widget,
@@ -168,20 +155,47 @@ static void icon_scroll_size_allocate_cb(GtkWidget 	*widget,
 					 MimeView 	*mimeview);
 static MimeInfo *mimeview_get_part_to_use(MimeView *mimeview);
 
-static void mimeview_launch_cb(MimeView *mimeview)
+static void mimeview_launch_cb(GtkAction *action, gpointer data)
 {
+	MimeView *mimeview = (MimeView *)data;
 	mimeview_launch(mimeview, mimeview_get_part_to_use(mimeview));
 }
-static GtkItemFactoryEntry mimeview_popup_entries[] =
+
+static void mimeview_open_with_cb(GtkAction *action, gpointer data)
 {
-	{N_("/_Open (l)"),		  NULL, mimeview_launch_cb,	  0, NULL},
+	mimeview_open_with((MimeView *)data);
+}
+
+static void mimeview_display_as_text_cb(GtkAction *action, gpointer data)
+{
+	mimeview_display_as_text((MimeView *)data);
+}
+
+static void mimeview_save_as_cb(GtkAction *action, gpointer data)
+{
+	mimeview_save_as((MimeView *)data);
+}
+
+static void mimeview_save_all_cb(GtkAction *action, gpointer data)
+{
+	mimeview_save_all((MimeView *)data);
+}
+
+static void mimeview_select_next_part_cb(GtkAction *action, gpointer data)
+{
+	mimeview_select_next_part((MimeView *)data);
+}
+
+static GtkActionEntry mimeview_menu_actions[] = {
+	{ "MimeView", NULL, "MimeView" },
+	{ "MimeView/Open", NULL, N_("_Open (l)"), NULL, "Open MIME part", G_CALLBACK(mimeview_launch_cb) },
 #ifndef MAEMO
-	{N_("/Open _with (o)..."),	  NULL, mimeview_open_with,	  0, NULL},
+	{ "MimeView/OpenWith", NULL, N_("Open _with (o)..."), NULL, "Open MIME part with...", G_CALLBACK(mimeview_open_with_cb) },
 #endif
-	{N_("/_Display as text (t)"), 	  NULL, mimeview_display_as_text, 0, NULL},
-	{N_("/_Save as (y)..."),	  NULL, mimeview_save_as,	  0, NULL},
-	{N_("/Save _all..."),	 	  NULL, mimeview_save_all,	  0, NULL},
-	{N_("/Next part (a)"),	 	  NULL, mimeview_select_next_part,0, NULL},
+	{ "MimeView/DisplayAsText", NULL, N_("_Display as text (t)"), NULL, "Display as text", G_CALLBACK(mimeview_display_as_text_cb) },
+	{ "MimeView/SaveAs", NULL, N_("_Save as (y)..."), NULL, "Save as", G_CALLBACK(mimeview_save_as_cb) },
+	{ "MimeView/SaveAll", NULL, N_("Save _all..."), NULL, "Save all parts", G_CALLBACK(mimeview_save_all_cb) },
+	{ "MimeView/NextPart", NULL, N_("Next part (a)"), NULL, "Next part", G_CALLBACK(mimeview_select_next_part_cb) }
 };
 
 static GtkTargetEntry mimeview_mime_types[] =
@@ -236,12 +250,12 @@ MimeView *mimeview_create(MainWindow *mainwin)
 	GtkWidget *arrow;
 	GtkWidget *scrollbutton;
 	GtkWidget *hbox;
-	GtkTooltips *tooltips;
-	GtkItemFactory *popupfactory;
+	GtkUIManager *gui_manager = gtkut_ui_manager();
+	GtkActionGroup *actions;
 	NoticeView *siginfoview;
 	gchar *titles[N_MIMEVIEW_COLS];
-	gint n_entries;
 	gint i;
+	CLAWS_TIP_DECL();
 
 	if (!hand_cursor)
 		hand_cursor = gdk_cursor_new(GDK_HAND2);
@@ -261,14 +275,14 @@ MimeView *mimeview_create(MainWindow *mainwin)
 
 	ctree = gtk_sctree_new_with_titles(N_MIMEVIEW_COLS, 0, titles);
 	gtk_widget_show(ctree);
-	gtk_clist_set_selection_mode(GTK_CLIST(ctree), GTK_SELECTION_BROWSE);
-	gtk_ctree_set_line_style(GTK_CTREE(ctree), GTK_CTREE_LINES_NONE);
-	gtk_clist_set_column_justification(GTK_CLIST(ctree), COL_SIZE,
+	gtk_cmclist_set_selection_mode(GTK_CMCLIST(ctree), GTK_SELECTION_BROWSE);
+	gtk_cmctree_set_line_style(GTK_CMCTREE(ctree), GTK_CMCTREE_LINES_NONE);
+	gtk_cmclist_set_column_justification(GTK_CMCLIST(ctree), COL_SIZE,
 					   GTK_JUSTIFY_RIGHT);
-	gtk_clist_set_column_width(GTK_CLIST(ctree), COL_MIMETYPE, 240);
-	gtk_clist_set_column_width(GTK_CLIST(ctree), COL_SIZE, 90);
+	gtk_cmclist_set_column_width(GTK_CMCLIST(ctree), COL_MIMETYPE, 240);
+	gtk_cmclist_set_column_width(GTK_CMCLIST(ctree), COL_SIZE, 90);
 	for (i = 0; i < N_MIMEVIEW_COLS; i++)
-		GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(ctree)->column[i].button,
+		GTK_WIDGET_UNSET_FLAGS(GTK_CMCLIST(ctree)->column[i].button,
 				       GTK_CAN_FOCUS);
 	gtk_container_add(GTK_CONTAINER(scrolledwin), ctree);
 
@@ -333,11 +347,28 @@ MimeView *mimeview_create(MainWindow *mainwin)
 	ctree_mainbox = gtk_hbox_new(FALSE, 0);	
 	gtk_box_pack_start(GTK_BOX(ctree_mainbox), scrolledwin, TRUE, TRUE, 0);
 
- 	n_entries = sizeof(mimeview_popup_entries) /
- 		sizeof(mimeview_popup_entries[0]);
- 	popupmenu = menu_create_items(mimeview_popup_entries, n_entries,
- 				      "<MimeView>", &popupfactory, mimeview);
-	tooltips = gtk_tooltips_new();
+	actions = cm_menu_create_action_group("MimeView", mimeview_menu_actions,
+			G_N_ELEMENTS(mimeview_menu_actions), (gpointer)mimeview);
+
+	MENUITEM_ADDUI("/Menus/", "MimeView", "MimeView", GTK_UI_MANAGER_MENU);
+	MENUITEM_ADDUI("/Menus/MimeView/", "Open", "MimeView/Open",
+			GTK_UI_MANAGER_MENUITEM);
+#ifndef MAEMO
+	MENUITEM_ADDUI("/Menus/MimeView/", "OpenWith", "MimeView/OpenWith",
+			GTK_UI_MANAGER_MENUITEM);
+#endif
+	MENUITEM_ADDUI("/Menus/MimeView/", "DisplayAsText", "MimeView/DisplayAsText",
+			GTK_UI_MANAGER_MENUITEM);
+	MENUITEM_ADDUI("/Menus/MimeView/", "SaveAs", "MimeView/SaveAs",
+			GTK_UI_MANAGER_MENUITEM);
+	MENUITEM_ADDUI("/Menus/MimeView/", "SaveAll", "MimeView/SaveAll",
+			GTK_UI_MANAGER_MENUITEM);
+	MENUITEM_ADDUI("/Menus/MimeView/", "NextPart", "MimeView/NextPart",
+			GTK_UI_MANAGER_MENUITEM);
+
+	popupmenu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(
+				gtk_ui_manager_get_widget(gui_manager, "/Menus/MimeView")) );
+
 
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_widget_show(vbox);
@@ -349,7 +380,6 @@ MimeView *mimeview_create(MainWindow *mainwin)
 
 	paned = gtk_vpaned_new();
 	gtk_widget_show(paned);
-	gtk_paned_set_gutter_size(GTK_PANED(paned), 0);
 	gtk_paned_pack1(GTK_PANED(paned), ctree_mainbox, FALSE, TRUE);
 	gtk_paned_pack2(GTK_PANED(paned), vbox, TRUE, TRUE);
 	
@@ -368,7 +398,6 @@ MimeView *mimeview_create(MainWindow *mainwin)
 	mimeview->ctree         = ctree;
 	mimeview->mime_notebook = mime_notebook;
 	mimeview->popupmenu     = popupmenu;
-	mimeview->popupfactory  = popupfactory;
 	mimeview->type          = -1;
 	mimeview->ctree_mainbox = ctree_mainbox;
 	mimeview->icon_scroll   = icon_scroll;
@@ -376,7 +405,9 @@ MimeView *mimeview_create(MainWindow *mainwin)
 	mimeview->icon_mainbox  = icon_mainbox;
 	mimeview->icon_count    = 0;
 	mimeview->mainwin       = mainwin;
-	mimeview->tooltips      = tooltips;
+#if !(GTK_CHECK_VERSION(2,12,0))
+	mimeview->tooltips      = tips;
+#endif
 	mimeview->oldsize       = 60;
 	mimeview->mime_toggle   = mime_toggle;
 	mimeview->siginfoview	= siginfoview;
@@ -410,8 +441,8 @@ static gboolean any_part_is_signed(MimeInfo *mimeinfo)
 void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 			   const gchar *file)
 {
-	GtkCTree *ctree = GTK_CTREE(mimeview->ctree);
-	GtkCTreeNode *node;
+	GtkCMCTree *ctree = GTK_CMCTREE(mimeview->ctree);
+	GtkCMCTreeNode *node;
 
 	mimeview_clear(mimeview);
 
@@ -437,11 +468,11 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 	g_signal_handlers_unblock_by_func(G_OBJECT(ctree),
 					  mimeview_selected, mimeview);
 
-	node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+	node = GTK_CMCTREE_NODE(GTK_CMCLIST(ctree)->row_list);
 	if (node) {
-		gtk_ctree_select(ctree, node);
+		gtk_cmctree_select(ctree, node);
 		icon_list_toggle_by_mime_info
-			(mimeview, gtk_ctree_node_get_row_data(ctree, node));
+			(mimeview, gtk_cmctree_node_get_row_data(ctree, node));
 		gtkut_ctree_set_focus_row(ctree, node);
 	}
 }
@@ -497,16 +528,16 @@ void mimeview_destroy(MimeView *mimeview)
 
 MimeInfo *mimeview_get_selected_part(MimeView *mimeview)
 {
-	return gtk_ctree_node_get_row_data
-		(GTK_CTREE(mimeview->ctree), mimeview->opened);
+	return gtk_cmctree_node_get_row_data
+		(GTK_CMCTREE(mimeview->ctree), mimeview->opened);
 }
 
 gint mimeview_get_selected_part_num(MimeView *mimeview)
 {
 	gint i = 0;
-	GList *rows = GTK_CLIST(mimeview->ctree)->row_list;
+	GList *rows = GTK_CMCLIST(mimeview->ctree)->row_list;
 	while (rows) {
-		if (mimeview->opened == GTK_CTREE_NODE(rows))
+		if (mimeview->opened == GTK_CMCTREE_NODE(rows))
 			return i;
 		i++;
 		rows = rows->next;
@@ -518,16 +549,16 @@ void mimeview_select_part_num(MimeView *mimeview, gint i)
 {
 	if (i < 0)
 		return;
-	gtk_clist_unselect_all(GTK_CLIST(mimeview->ctree));
-	gtk_clist_select_row(GTK_CLIST(mimeview->ctree), i, -1);
-	gtkut_clist_set_focus_row(GTK_CLIST(mimeview->ctree), i);
+	gtk_cmclist_unselect_all(GTK_CMCLIST(mimeview->ctree));
+	gtk_cmclist_select_row(GTK_CMCLIST(mimeview->ctree), i, -1);
+	gtkut_clist_set_focus_row(GTK_CMCLIST(mimeview->ctree), i);
 }
 
 static void mimeview_set_multipart_tree(MimeView *mimeview,
 					MimeInfo *mimeinfo,
-					GtkCTreeNode *parent)
+					GtkCMCTreeNode *parent)
 {
-	GtkCTreeNode *node;
+	GtkCMCTreeNode *node;
 
 	g_return_if_fail(mimeinfo != NULL);
 
@@ -570,12 +601,12 @@ static const gchar *get_part_description(MimeInfo *partinfo)
 		return get_part_name(partinfo);
 }
 
-static GtkCTreeNode *mimeview_append_part(MimeView *mimeview,
+static GtkCMCTreeNode *mimeview_append_part(MimeView *mimeview,
 					  MimeInfo *partinfo,
-					  GtkCTreeNode *parent)
+					  GtkCMCTreeNode *parent)
 {
-	GtkCTree *ctree = GTK_CTREE(mimeview->ctree);
-	GtkCTreeNode *node;
+	GtkCMCTree *ctree = GTK_CMCTREE(mimeview->ctree);
+	GtkCMCTreeNode *node;
 	static gchar content_type[64];
 	gchar *str[N_MIMEVIEW_COLS];
 
@@ -595,7 +626,7 @@ static GtkCTreeNode *mimeview_append_part(MimeView *mimeview,
 	node = gtk_sctree_insert_node(ctree, parent, NULL, str, 0,
 				     NULL, NULL, NULL, NULL,
 				     FALSE, TRUE);
-	gtk_ctree_node_set_row_data(ctree, node, partinfo);
+	gtk_cmctree_node_set_row_data(ctree, node, partinfo);
 
 	return node;
 }
@@ -632,16 +663,16 @@ static MimeViewer *get_viewer_for_content_type(MimeView *mimeview, const gchar *
 	GSList *cur;
 	MimeViewerFactory *factory = NULL;
 	MimeViewer *viewer = NULL;
+	gchar *real_contenttype = NULL;
 
 /*
  * FNM_CASEFOLD is a GNU extension
- * if its not defined copy the string to the stack and
- * convert the copy to lower case
  */
 #ifndef FNM_CASEFOLD
 #define FNM_CASEFOLD 0
-	Xstrdup_a(content_type, content_type, return NULL);
-	g_strdown((gchar *)content_type);
+	real_contenttype = g_utf8_strdown((gchar *)content_type, -1);
+#else
+	real_contenttype = g_strdup(content_type);
 #endif
 	
 	for (cur = mimeviewer_factories; cur != NULL; cur = g_slist_next(cur)) {
@@ -649,7 +680,7 @@ static MimeViewer *get_viewer_for_content_type(MimeView *mimeview, const gchar *
 		gint i = 0;
 
 		while (curfactory->content_types[i] != NULL) {
-			if(!fnmatch(curfactory->content_types[i], content_type, FNM_CASEFOLD)) {
+			if(!fnmatch(curfactory->content_types[i], real_contenttype, FNM_CASEFOLD)) {
 				debug_print("%s\n", curfactory->content_types[i]);
 				factory = curfactory;
 				break;
@@ -659,6 +690,7 @@ static MimeViewer *get_viewer_for_content_type(MimeView *mimeview, const gchar *
 		if (factory != NULL)
 			break;
 	}
+	g_free(real_contenttype);
 	if (factory == NULL)
 		return NULL;
 
@@ -712,6 +744,11 @@ gboolean mimeview_show_part(MimeView *mimeview, MimeInfo *partinfo)
 {
 	MimeViewer *viewer;
 	
+	if (mimeview->messageview->partial_display_shown) {
+		noticeview_hide(mimeview->messageview->noticeview);
+		mimeview->messageview->partial_display_shown = FALSE;
+	}
+
 	viewer = get_viewer_for_mimeinfo(mimeview, partinfo);
 	if (viewer == NULL) {
 		if (mimeview->mimeviewer != NULL)
@@ -766,7 +803,7 @@ static void mimeview_change_view_type(MimeView *mimeview, MimeViewType type)
 
 void mimeview_clear(MimeView *mimeview)
 {
-	GtkCList *clist = NULL;
+	GtkCMCList *clist = NULL;
 	
 	if (!mimeview)
 		return;
@@ -774,11 +811,11 @@ void mimeview_clear(MimeView *mimeview)
 	if (g_slist_find(mimeviews, mimeview) == NULL)
 		return;
 	
-	clist = GTK_CLIST(mimeview->ctree);
+	clist = GTK_CMCLIST(mimeview->ctree);
 	
 	noticeview_hide(mimeview->siginfoview);
 
-	gtk_clist_clear(clist);
+	gtk_cmclist_clear(clist);
 	textview_clear(mimeview->textview);
 	if (mimeview->mimeviewer != NULL)
 		mimeview->mimeviewer->clear_viewer(mimeview->mimeviewer);
@@ -1093,9 +1130,9 @@ static void mimeview_check_signature(MimeView *mimeview)
 static void redisplay_email(GtkWidget *widget, gpointer user_data)
 {
 	MimeView *mimeview = (MimeView *) user_data;
-	GtkCTreeNode *node = mimeview->opened;
+	GtkCMCTreeNode *node = mimeview->opened;
 	mimeview->opened = NULL;
-	mimeview_selected(GTK_CTREE(mimeview->ctree), node, 0, mimeview);
+	mimeview_selected(GTK_CMCTREE(mimeview->ctree), node, 0, mimeview);
 }
 
 static void display_full_info_cb(GtkWidget *widget, gpointer user_data)
@@ -1152,16 +1189,16 @@ static void update_signature_info(MimeView *mimeview, MimeInfo *selected)
 	noticeview_show(mimeview->siginfoview);
 }
 
-static void mimeview_selected(GtkCTree *ctree, GtkCTreeNode *node, gint column,
+static void mimeview_selected(GtkCMCTree *ctree, GtkCMCTreeNode *node, gint column,
 			      MimeView *mimeview)
 {
 	MimeInfo *partinfo;
 	if (mimeview->opened == node) return;
 	mimeview->opened = node;
 	mimeview->spec_part = NULL;
-	gtk_ctree_node_moveto(ctree, node, -1, 0.5, 0);
+	gtk_cmctree_node_moveto(ctree, node, -1, 0.5, 0);
 
-	partinfo = gtk_ctree_node_get_row_data(ctree, node);
+	partinfo = gtk_cmctree_node_get_row_data(ctree, node);
 	if (!partinfo) return;
 
 	/* ungrab the mouse event */
@@ -1212,17 +1249,17 @@ static void mimeview_start_drag(GtkWidget *widget, gint button,
 static gint mimeview_button_pressed(GtkWidget *widget, GdkEventButton *event,
 				    MimeView *mimeview)
 {
-	GtkCList *clist = GTK_CLIST(widget);
+	GtkCMCList *clist = GTK_CMCLIST(widget);
 	gint row, column;
 
 	if (!event) return FALSE;
 
 	if (event->button == 2 || event->button == 3) {
-		if (!gtk_clist_get_selection_info(clist, event->x, event->y,
+		if (!gtk_cmclist_get_selection_info(clist, event->x, event->y,
 						  &row, &column))
 			return FALSE;
-		gtk_clist_unselect_all(clist);
-		gtk_clist_select_row(clist, row, column);
+		gtk_cmclist_unselect_all(clist);
+		gtk_cmclist_select_row(clist, row, column);
 		gtkut_clist_set_focus_row(clist, row);
 	}
 	part_button_pressed(mimeview, event, mimeview_get_selected_part(mimeview));
@@ -1261,19 +1298,15 @@ static gboolean part_button_pressed(MimeView *mimeview, GdkEventButton *event,
 		if (partinfo && (partinfo->type == MIMETYPE_MESSAGE ||
 				 partinfo->type == MIMETYPE_IMAGE ||
 				 partinfo->type == MIMETYPE_MULTIPART))
-			menu_set_sensitive(mimeview->popupfactory,
-					   "/Display as text (t)", FALSE);
+			cm_menu_set_sensitive("MimeView/DisplayAsText", FALSE);
 		else
-			menu_set_sensitive(mimeview->popupfactory,
-					   "/Display as text (t)", TRUE);
+			cm_menu_set_sensitive("MimeView/DisplayAsText", TRUE);
 		if (partinfo &&
 		    partinfo->type == MIMETYPE_APPLICATION &&
 		    !g_ascii_strcasecmp(partinfo->subtype, "octet-stream"))
-			menu_set_sensitive(mimeview->popupfactory,
-					   "/Open (l)", FALSE);
+			cm_menu_set_sensitive("MimeView/Open", FALSE);
 		else
-			menu_set_sensitive(mimeview->popupfactory,
-					   "/Open (l)", TRUE);
+			cm_menu_set_sensitive("MimeView/Open", TRUE);
 
 		g_object_set_data(G_OBJECT(mimeview->popupmenu),
 				  "pop_partinfo", partinfo);
@@ -1297,17 +1330,17 @@ void mimeview_pass_key_press_event(MimeView *mimeview, GdkEventKey *event)
 
 static void mimeview_select_next_part(MimeView *mimeview)
 {
-	GtkCTree *ctree = GTK_CTREE(mimeview->ctree);
-	GtkCTreeNode *node = mimeview->opened;
+	GtkCMCTree *ctree = GTK_CMCTREE(mimeview->ctree);
+	GtkCMCTreeNode *node = mimeview->opened;
 	MimeInfo *partinfo = NULL;
 	
 skip:
-	node = GTK_CTREE_NODE_NEXT(node);
+	node = GTK_CMCTREE_NODE_NEXT(node);
 	if (!node)
-		node = gtk_ctree_node_nth(GTK_CTREE(ctree), 0);
+		node = gtk_cmctree_node_nth(GTK_CMCTREE(ctree), 0);
 
 	if (node) {
-		partinfo = gtk_ctree_node_get_row_data(ctree, node);
+		partinfo = gtk_cmctree_node_get_row_data(ctree, node);
 		if (partinfo->type == MIMETYPE_MULTIPART)
 			goto skip;
 		gtk_sctree_unselect_all(GTK_SCTREE(ctree));
@@ -1318,17 +1351,17 @@ skip:
 
 static void mimeview_select_prev_part(MimeView *mimeview)
 {
-	GtkCTree *ctree = GTK_CTREE(mimeview->ctree);
-	GtkCTreeNode *node = mimeview->opened;
+	GtkCMCTree *ctree = GTK_CMCTREE(mimeview->ctree);
+	GtkCMCTreeNode *node = mimeview->opened;
 	MimeInfo *partinfo = NULL;
 	
 skip:
-	node = GTK_CTREE_NODE_PREV(node);
+	node = GTK_CMCTREE_NODE_PREV(node);
 	if (!node)
-		node = gtk_ctree_node_nth(GTK_CTREE(ctree), GTK_CLIST(ctree)->rows - 1);
+		node = gtk_cmctree_node_nth(GTK_CMCTREE(ctree), GTK_CMCLIST(ctree)->rows - 1);
 
 	if (node) {
-		partinfo = gtk_ctree_node_get_row_data(ctree, node);
+		partinfo = gtk_cmctree_node_get_row_data(ctree, node);
 		if (partinfo->type == MIMETYPE_MULTIPART)
 			goto skip;
 		gtk_sctree_unselect_all(GTK_SCTREE(ctree));
@@ -1348,7 +1381,7 @@ static gint mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 				 MimeView *mimeview)
 {
 	SummaryView *summaryview;
-	GtkCTree *ctree = GTK_CTREE(widget);
+	GtkCMCTree *ctree = GTK_CMCTREE(widget);
 
 	if (!event) return FALSE;
 	if (!mimeview->opened) return FALSE;
@@ -1558,7 +1591,7 @@ static gchar *mimeview_get_filename_for_part(MimeInfo *partinfo,
 	subst_for_filename(filename);
 
 	fullname = g_strconcat
-		(basedir, G_DIR_SEPARATOR_S, (filename[0] == G_DIR_SEPARATOR)
+		(basedir, G_DIR_SEPARATOR_S, (g_path_is_absolute(filename))
 		 ? &filename[1] : filename, NULL);
 
 	g_free(filename);
@@ -2013,11 +2046,11 @@ static gboolean icon_clicked_cb (GtkWidget *button, GdkEventButton *event, MimeV
 
 static void icon_selected (MimeView *mimeview, gint num, MimeInfo *partinfo)
 {
-	GtkCTreeNode *node;
-	node = gtk_ctree_find_by_row_data(GTK_CTREE(mimeview->ctree), NULL, partinfo);
+	GtkCMCTreeNode *node;
+	node = gtk_cmctree_find_by_row_data(GTK_CMCTREE(mimeview->ctree), NULL, partinfo);
 
 	if (node)
-		gtk_ctree_select(GTK_CTREE(mimeview->ctree), node);
+		gtk_cmctree_select(GTK_CMCTREE(mimeview->ctree), node);
 }		
 
 void mimeview_select_mimepart_icon(MimeView *mimeview, MimeInfo *partinfo)
@@ -2131,6 +2164,9 @@ static void icon_list_append_icon (MimeView *mimeview, MimeInfo *mimeinfo)
 	MimeInfo *partinfo;
 	MimeInfo *siginfo = NULL;
 	MimeInfo *encrypted = NULL;
+#if !(GTK_CHECK_VERSION(2,12,0))
+	GtkTooltips *tips = mimeview->tooltips;
+#endif
 	
 	vbox = mimeview->icon_vbox;
 	mimeview->icon_count++;
@@ -2266,7 +2302,7 @@ static void icon_list_append_icon (MimeView *mimeview, MimeInfo *mimeinfo)
 	}
 	g_free(sigshort);
 
-	gtk_tooltips_set_tip(mimeview->tooltips, button, tip, NULL);
+	CLAWS_SET_TIP(button, tip);
 	g_free(tip);
 	gtk_widget_show_all(button);
 	gtk_drag_source_set(button, GDK_BUTTON1_MASK|GDK_BUTTON3_MASK, 
@@ -2299,11 +2335,10 @@ static void icon_list_clear (MimeView *mimeview)
 	GList     *child;
 	GtkAdjustment *adj;
 		
-	child = gtk_container_children(GTK_CONTAINER(mimeview->icon_vbox));
+	child = gtk_container_get_children(GTK_CONTAINER(mimeview->icon_vbox));
 	for (; child != NULL; child = g_list_next(child)) {
 		gtkut_container_remove(GTK_CONTAINER(mimeview->icon_vbox), 
 				       GTK_WIDGET(child->data));
-		gtk_widget_destroy(GTK_WIDGET(child->data));
 	}
 	g_list_free(child);
 	mimeview->icon_count = 0;
@@ -2365,7 +2400,7 @@ static void icon_list_toggle_by_mime_info (MimeView	*mimeview,
 {
 	GList *child;
 	
-	child = gtk_container_children(GTK_CONTAINER(mimeview->icon_vbox));
+	child = gtk_container_get_children(GTK_CONTAINER(mimeview->icon_vbox));
 	for (; child != NULL; child = g_list_next(child)) {
 		if (!GTK_IS_EVENT_BOX(child->data))
 			continue;
@@ -2389,13 +2424,13 @@ static void icon_list_toggle_by_mime_info (MimeView	*mimeview,
 static gint mime_toggle_button_cb(GtkWidget *button, GdkEventButton *event,
 				    MimeView *mimeview)
 {
-	gtk_widget_ref(button); 
+	g_object_ref(button); 
 
 	mimeview_leave_notify(button, NULL, NULL);
 
 	mimeview->ctree_mode = !mimeview->ctree_mode;
 	if (mimeview->ctree_mode) {
-		gtk_arrow_set(GTK_ARROW(GTK_BIN(button)->child), GTK_ARROW_RIGHT, 
+		gtk_arrow_set(GTK_ARROW(gtk_bin_get_child(GTK_BIN((button)))), GTK_ARROW_RIGHT, 
 					GTK_SHADOW_NONE);
 		gtk_widget_hide(mimeview->icon_mainbox);
 		gtk_widget_show(mimeview->ctree_mainbox);
@@ -2405,9 +2440,8 @@ static gint mime_toggle_button_cb(GtkWidget *button, GdkEventButton *event,
 					button);
 		gtk_box_pack_end(GTK_BOX(mimeview->ctree_mainbox), 
 				   button, FALSE, FALSE, 0);
-		gtk_paned_set_gutter_size(GTK_PANED(mimeview->paned), 6);
 	} else {
-		gtk_arrow_set(GTK_ARROW(GTK_BIN(button)->child), GTK_ARROW_LEFT, 
+		gtk_arrow_set(GTK_ARROW(gtk_bin_get_child(GTK_BIN((button)))), GTK_ARROW_LEFT, 
 			      GTK_SHADOW_NONE);
 		mimeview->oldsize = mimeview->ctree_mainbox->allocation.height;
 		gtk_widget_hide(mimeview->ctree_mainbox);
@@ -2421,12 +2455,11 @@ static gint mime_toggle_button_cb(GtkWidget *button, GdkEventButton *event,
 		gtk_box_reorder_child(GTK_BOX(button->parent), button, 0);
 		if (mimeview->opened)
 			icon_list_toggle_by_mime_info
-				(mimeview, gtk_ctree_node_get_row_data(GTK_CTREE(mimeview->ctree), 
+				(mimeview, gtk_cmctree_node_get_row_data(GTK_CMCTREE(mimeview->ctree), 
 								       mimeview->opened));
 
-		gtk_paned_set_gutter_size(GTK_PANED(mimeview->paned), 0);
 	}
-	gtk_widget_unref(button);
+	g_object_unref(button);
 	return TRUE;
 }
 
